%%%% READ SOUNDING DATA FROM LATEST ETL FORMAT...
%%%% COMPOSITE TIME-HEIGHT DISPLAY ....
%%%% PK
%%%% DEW 12/11/2004
%%%% ES  04/16/2005  

clear all;
fclose all;
warning off MATLAB:divideByZero  % turn off warning message

pathdata = 'C:\THESIS\DATA\Stratus04\sondes\data\';
eval(['cd ', pathdata]);

aa = dir('*.txt');
k = length(aa);
[num] = num2str(k);
disp('Number of rawinsonde edt_WTEC*.txt files : '); disp([num]);

%%%%%%%%% starting file is ws %%%%%%%%%%%%%%%%%%%
ws=1;
we=k;
%%%%%%%%%%%%%%%%%% last file is we=k %%%%%%%%%%%%%%%

for w = ws:we 
    w
    [numf] = num2str(w);

    disp('# of edt_WTEC*.txt file in process :'); disp ( [numf]);
    sounding_filename = aa(w).name

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% READ THE STRATUS 04 EDT FILE %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    year(w)   = 2004;
    month(w)  = str2num(sounding_filename(11:12));
    day (w)   = str2num(sounding_filename(13:14));
    hour(w)   = str2num(sounding_filename(15:16));
    minutes(w)= str2num(sounding_filename(17:18));

    %%%% COMMON TIME COORDINATE %%%%%
    
    %jul_day_sonde(w) = datenum(year(w),month(w),day(w),hour(w),minutes(w),00)-datenum(2003,12,31,00,00,00);
    day_Dec(w) = day(w)+hour(w)/24+minutes(w)/(24*60); 
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    fid1=fopen(sounding_filename,'r');

    if fid1 == -1
        error(['Cannot read ' sounding_filename])
    end
    
    dh = 19;
    if w >2
       dh = 20;
    end
    
    %%% read overhead...
    for i=1:dh;
        dummy=fgetl(fid1);
    end
    
    nhts = 9999;
    
    % Dan uses nhts=2000 (so up to ~18800 m) for sounding #50 (16 Dec 2004 1152), most probably
    % to skip the last 2 lines of data
    
    if w == 50
      nhts = 2100;   %% OK with 2100
    end
    
    k=1;
    %while ~feof(fid1)
    for k = 1:nhts
        dout=fgetl(fid1);
        spout=fgetl(fid1); % rad in extra space between each data line

        time(k,w) = str2num(dout(1:5));   % Time
        H(k,w)    = str2num(dout(6:14));  % Height (m)
        P(k,w)    = str2num(dout(16:24)); % Pressure (mb)
        T(k,w)    = str2num(dout(26:33)); % Temp (C)
        Td(k,w)   = str2num(dout(35:42)); % Dewpoint (C)
        RH(k,w)   = nan;
        %Td(k,w)  = nan;  Dan had this "on", but it leads to no values of Td
        spd(k,w)  = nan;
        dir(k,w)  = nan;
        
        %%%% Take into account a possible non-existence of data (////) %%%%
        
        if dout(50) == '/'
            if dout(67) ~= '/'
                dir(k,w)=str2num(dout(65:69)); %DIR
                spd(k,w)=str2num(dout(58:63)); %SPD
            end
        else
            RH(k,w)=str2num(dout(44:49)); % %RH
            if dout(64) ~= '/'
                if k <= 5000
                    dir(k,w)=str2num(dout(60:64)); %DIR
                end
                if k > 5000
                    dir(k,w)=str2num(dout(60:65)); %DIR
                end
                spd(k,w)=str2num(dout(51:58)); % SPD
            end
            teof = feof(fid1);

            if teof ~= 0
               break
            end
        end

    end  %% end for k
    k;
end  %% end for w


%%%% Now QC and Interpolation %%%%
%%%  FIRST QUALITY CONTROL....

h_int = [10:10:20000];

for i=ws:we;
    g = find(H(:,i)>10 & H(:,i)<=20000);
    
    gk = find(diff(H(g,i))>0);   %%%% Get values with positive difference %%%%
    c = g(gk); clear g; g=c; clear c gk;
    
    gk = find(diff(H(g,i))>0);   %%%% Repeat this 4 times %%%%
    c = g(gk); clear g; g=c; clear c gk;
    
    gk = find(diff(H(g,i))>0);   
    c = g(gk); clear g; g=c; clear c gk;
    
    gk = find(diff(H(g,i))>0);  
    c = g(gk); clear g; g=c; clear c gk;
    
    f=find(T(g,i)>-273);
    if isempty(f)==0
        t_int1(1:length(h_int),i)     =  interp1(H(g(f),i),T(g(f),i),h_int)'; clear f;
    end
    
    f=find(Td(g,i)>-273);
    if isempty(f)==0
        dew_int1(1:length(h_int),i)   =  interp1(H(g(f),i),Td(g(f),i),h_int)'; clear f;
    end
    
    f=find(RH(g,i)>0);    
    if isempty(f)==0
        rh_int1(1:length(h_int),i)    =  interp1(H(g(f),i),RH(g(f),i),h_int)'; clear f;
    end
    
    f=find(P(g,i)>0);     
    if isempty(f)==0
        press_int1(1:length(h_int),i) =  interp1(H(g(f),i),P(g(f),i),h_int)'; clear f;
    end
    
    f=find(dir(g,i)>0);   
    if isempty(f)==0
        dir_int1(1:length(h_int),i)   =  interp1(H(g(f),i),dir(g(f),i),h_int)'; clear f;
    end
    
    f=find(spd(g,i)>0);   
    if isempty(f)==0
        spd_int1(1:length(h_int),i)   =  interp1(H(g(f),i),spd(g(f),i),h_int)'; clear f;
    end

    % Put surface values in 10 m ht interpolated arrays
    t_int1(1,i)     = T(1,i);
    dew_int1(1,i)   = Td(1,i);
    rh_int1(1,i)    = RH(1,i);
    press_int1(1,i) = P(1,i);
    spd_int1(1,i)   = spd(1,i);
    dir_int1(1,i)   = dir(1,i);
    
    clear g;
end

% Sounding #50 (041216-1152): Use NaN for all RH,Td values, due to intermittent periods of wrong (low) values 

rh_int1(:,50) = nan;
dew_int1(:,50) = nan;

% Sounding #65 (041220-0457): Use NaN for initial (2:120) spd/dir values , due to //// values 

spd_int1(2:120,65) = nan;
dir_int1(2:120,65) = nan;

% Correction of initial values for sounding #72 (041221-1858)

t_int1(1,72) = nan;
dew_int1(1,72) = nan;
press_int1(1,72) = nan;
rh_int1(1,72) = nan;
spd_int1(1,72) = nan;
dir_int1(1,72) = nan;


%%%%%%% Post-processing in order to create a permanent .mat file, suitable for the 
%%%%%%% code sondes_graphs.m - Basically putting NaN values in missing sondes

mday(1:31)=day_Dec(1:31);
mday(32)=13.125;
mday(33:79)=day_Dec(32:78);

clear day day_Dec hour minutes month year T Td P RH spd dir
clear aa dh dir dout dummy fid1 fid2 i k m nhts num* pathdata sound* spout teof temp w*

h_all(:,1:31)=H(:,1:31);
h_all(:,32)=nan;
h_all(:,33:79)=H(:,32:78);
clear H

time_all(:,1:31)=time(:,1:31);
time_all(:,32)=nan;
time_all(:,33:79)=time(:,32:78);
clear time

press_int(:,1:31)=press_int1(:,1:31);
press_int(:,32)=nan;
press_int(:,33:79)=press_int1(:,32:78);
clear press_int1

rh_int(:,1:31)=rh_int1(:,1:31);
rh_int(:,32)=nan;
rh_int(:,33:79)=rh_int1(:,32:78);
clear rh_int1

t_int(:,1:31)=t_int1(:,1:31);
t_int(:,32)=nan;
t_int(:,33:79)=t_int1(:,32:78);
clear t_int1

dew_int(:,1:31)=dew_int1(:,1:31);
dew_int(:,32)=nan;
dew_int(:,33:79)=dew_int1(:,32:78);
clear dew_int1

dir_int(:,1:31)=dir_int1(:,1:31);
dir_int(:,32)=nan;
dir_int(:,33:79)=dir_int1(:,32:78);
clear dir_int1

spd_int(:,1:31)=spd_int1(:,1:31);
spd_int(:,32)=nan;
spd_int(:,33:79)=spd_int1(:,32:78);
clear spd_int1

clear ans


save C:\THESIS\Processing_Results\Stratus04\sondes\stratus04_sondes_reader 
%press_int t_int dew_int rh_int spd_int dir_int h_int day_Dec;

